package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.MetricHistDTO;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.feign.service.ControlFeignClients;
import net.wanji.opt.bo.AbnormalDetailBO;
import net.wanji.opt.bo.MetricsTurnBO;
import net.wanji.opt.bo.SceneMetricsDetailBO;
import net.wanji.opt.service.SceneEvaluateService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/28 16:38
 */
@Slf4j
@Service
public class SceneEvaluateServiceImpl implements SceneEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;
    private final ControlFeignClients controlFeignClients;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    SimpleDateFormat HOUR_SDF = new SimpleDateFormat("HH:mm");

    public SceneEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper, RunningEvaluateServiceImpl runningEvaluateService, @Qualifier("baseCrossSchemeMapper") BaseCrossSchemeMapper baseCrossSchemeMapper, @Qualifier("baseCrossSchedulesMapper") BaseCrossSchedulesMapper baseCrossSchedulesMapper, @Qualifier("baseCrossSchedulesPlanMapper") BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper, @Qualifier("net.wanji.feign.service.ControlFeignClients") ControlFeignClients controlFeignClients, @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper, CrossDirDataHistMapper crossDirDataHistMapper, CrossLaneDataHistMapper crossLaneDataHistMapper, CrossTurnDataHistMapper crossTurnDataHistMapper) {
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
        this.controlFeignClients = controlFeignClients;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
    }

    @Override
    public SceneEvaluateAbnormalDistributeVO abnormalDistribute(CrossIdAndStartEndDateBO bo) {
        String crossId = bo.getCrossId();
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();

        SceneEvaluateAbnormalDistributeVO vo = new SceneEvaluateAbnormalDistributeVO();
        List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList =
                buildTimeDistributionList(crossId, startDate, endDate);
        vo.setTimeDateDistributionList(buildFinalTimeDistributionList(timeDistributionList));

        setCounts(timeDistributionList, vo);
        return vo;
    }

    @Override
    public SceneEvaluateAbnormalDetailVO abnormalDetail(AbnormalDetailBO abnormalDetailBO) throws ParseException {
        String crossId = abnormalDetailBO.getCrossId();
        String hourMinute = abnormalDetailBO.getHourMinute();
        Date problemDate = abnormalDetailBO.getProblemDate();
        Integer status = abnormalDetailBO.getStatus();

        // 获取当前年份
        Calendar calendar = Calendar.getInstance();
        int currentYear = calendar.get(Calendar.YEAR);
        // 设置 problemDate 的年份为当前年份
        calendar.setTime(problemDate);
        calendar.set(Calendar.YEAR, currentYear);
        Date newDate = calendar.getTime();

        calendar.setTime(newDate);
        // 设置时间为当天开始时间（00:00:00）
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        int startOfDayTimestamp = (int) (calendar.getTimeInMillis() / 1000);
        // 设置时间为当天结束时间（23:59:59）
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        int endOfDayTimestamp = (int) (calendar.getTimeInMillis() / 1000);

        // 查询当天所有场景
        List<CrossDataHistPO> crossDataHistPOList =
                crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startOfDayTimestamp, endOfDayTimestamp);
        // 按照startTime分组并选择duration最长的一个。
        List<CrossDataHistPO> filteredList = crossDataHistPOList.stream()
                .filter(po -> Objects.equals(po.getStatus(), status))
                .collect(Collectors.groupingBy(CrossDataHistPO::getStartTime))
                .values().stream()
                .map(list -> list.stream().max(Comparator.comparingInt(CrossDataHistPO::getDuration)).get())
                .collect(Collectors.toList());

        // 进行startTime和endTime的格式化和分组。
        Map<String, List<CrossDataHistPO>> groupedData = filteredList.stream()
                .collect(Collectors.groupingBy(po -> {

                    // 计算startTime和endTime
                    Date startTime = po.getStartTime();
                    Date endTime = new Date(startTime.getTime() + po.getDuration() * 60 * 1000L);

                    // startTime向前取整至5分钟
                    calendar.setTime(startTime);
                    int minute = calendar.get(Calendar.MINUTE);
                    calendar.add(Calendar.MINUTE, -(minute % 5));
                    String startTimeStr = HOUR_SDF.format(calendar.getTime());

                    // endTime向后取整至5分钟
                    calendar.setTime(endTime);
                    minute = calendar.get(Calendar.MINUTE);
                    calendar.add(Calendar.MINUTE, 5 - (minute % 5));
                    String endTimeStr = HOUR_SDF.format(calendar.getTime());

                    return startTimeStr + "-" + endTimeStr;
                }));

        Date targetDate = HOUR_SDF.parse(hourMinute);
        // 获取当前年份，并设置targetDate的年份
        Calendar cal = Calendar.getInstance();
        cal.setTime(newDate);
        String[] split = hourMinute.split(":");
        cal.set(Calendar.HOUR, Integer.parseInt(split[0]));
        cal.set(Calendar.MINUTE, Integer.parseInt(split[1]));
        targetDate = cal.getTime();

        // 从groupedData中筛选与hourMinute相交的记录
        List<CrossDataHistPO> intersectingRecords = new ArrayList<>();

        for (Map.Entry<String, List<CrossDataHistPO>> entry : groupedData.entrySet()) {
            String key = entry.getKey();
            String[] times = key.split("-");

            Calendar startCal = Calendar.getInstance();
            Calendar endCal = Calendar.getInstance();
            startCal.setTime(newDate);
            endCal.setTime(newDate);
            String[] start = times[0].split(":");
            startCal.set(Calendar.HOUR, Integer.parseInt(start[0]));
            startCal.set(Calendar.MINUTE, Integer.parseInt(start[1]));
            String[] end = times[0].split(":");
            endCal.set(Calendar.HOUR, Integer.parseInt(end[0]));
            endCal.set(Calendar.MINUTE, Integer.parseInt(end[1]));
            Date startTime = startCal.getTime();
            Date endTime = endCal.getTime();

            // 检查相交条件
            if (!(targetDate.before(startTime) || targetDate.after(endTime))) {
                intersectingRecords.addAll(entry.getValue());
            }
        }

        if (ObjectUtil.isEmpty(intersectingRecords)) {
            throw new RuntimeException("该时段无异常事件");
        }

        CrossDataHistPO crossDataHistPO = intersectingRecords.get(0);
        SceneEvaluateAbnormalDetailVO vo = new SceneEvaluateAbnormalDetailVO();
        vo.setStatus(status);
        Date startTime = crossDataHistPO.getStartTime();
        Integer duration = crossDataHistPO.getDuration();
        // 计算startTime加上duration后的时间
        cal.setTime(startTime);
        cal.add(Calendar.MINUTE, duration);
        Date newEndTime = cal.getTime();

        List<CrossDataHistPO> sceneCrossDataHistPOList = getCrossDataHistPOList(startTime, newEndTime, crossId);

        vo.setStartTime(startTime);
        vo.setTimeline(getTimeline(startTime, newEndTime));
        // todo 暂时只有拥堵，后期扩展考虑增加字段
        if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            Integer delayTime = calcCrossDelayTime(sceneCrossDataHistPOList);
            Integer level = calcCongestionLevel(delayTime);
            vo.setLevel(level);
        }

        vo.setDuration(duration);
        if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            String congestionDirs = crossDataHistPO.getCongestionDirs();
            vo.setDirs(buildDirs(congestionDirs));
        }

        // 查找当前路口全部方案
        List<BaseCrossSchemePO> baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossId(crossId);
        // 找出当前计划
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String startDay = sdf.format(startTime);
        Integer planId = controlFeignClients.findPlanId(startTime, startDay, crossId);
        if (ObjectUtil.isEmpty(planId)) {
            throw new RuntimeException("当前时段无正在运行的计划");
        }
        // 获取时段信息
        List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossAndPlan(crossId, planId);
        // 根据 startTime 和 duration 查找相交的方案
        // 将Date转换为"HH:mm"字符串
        String newStartTimeStr = HOUR_SDF.format(startTime);
        String newEndTimeStr = HOUR_SDF.format(newEndTime);
        List<CrossSectionPO> intersectingSections = crossSectionPOList.stream()
                .filter(po -> !(po.getEndTime().compareTo(newStartTimeStr) < 0
                        || po.getStartTime().compareTo(newEndTimeStr) > 0))
                .collect(Collectors.toList());
        // 筛选baseCrossSchemePOList，找出id与intersectingSections中的schemeId相等的记录
        List<BaseCrossSchemePO> matchedSchemes = baseCrossSchemePOList.stream()
                .filter(baseScheme -> intersectingSections.stream()
                        .anyMatch(section -> section.getSchemeId().equals(baseScheme.getId())))
                .collect(Collectors.toList());
        vo.setPatternNames(buildPatternNames(matchedSchemes));

        Double sturation = calcCrossSaturation(sceneCrossDataHistPOList);
        String serviceLevel = CrossUtil.getServiceLevel(sturation);
        vo.setServiceLevel(serviceLevel);
        
        vo.setFlow(calcCrossFlow(sceneCrossDataHistPOList));
        
        if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            vo.setOverallMetricsList(buildCrossCongestionMetrics(
                    crossId, startTime, newEndTime, sceneCrossDataHistPOList));
        } else if (Objects.equals(status, CrossStatusEnum.UNBALANCE.getCode())) {
            vo.setOverallMetricsList(buildCrossUnbalanceMetrics(crossId, startTime, newEndTime, sceneCrossDataHistPOList));
        } else if (Objects.equals(status, CrossStatusEnum.SPILLOVER.getCode())) {
            vo.setOverallMetricsList(buildCrossSpilloverMetrics(sceneCrossDataHistPOList));
        }
        return vo;
    }

    @Override
    public List<SceneMetricsDetailVO> metricsDetail(SceneMetricsDetailBO bo)
            throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        String crossId = bo.getCrossId();
        Integer dir = bo.getDir();
        String turn = bo.getTurn();
        Integer laneSort = bo.getLaneSort();
        Date date = bo.getDate();
        Integer minutes = bo.getMinutes();
        List<String> metricCodes = bo.getMetricCodes();

        // 构造某一天开始和结束时间戳
        Date startDate = date;
        Date endDate = DateUtil.offsetDay(date, 1); // 包含最后一天
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);

        // 查询全部指标
        List<MetricHistDTO> metricHistDTOList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(laneSort)) {
            metricHistDTOList = crossLaneDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
        } else if (ObjectUtil.isNotEmpty(turn)) {
            metricHistDTOList = crossTurnDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
        } else if (ObjectUtil.isNotEmpty(dir)) {
            metricHistDTOList = crossDirDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
        } else { // 路口级别
            metricHistDTOList = crossDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
        }
        // 更新空放次数
        metricHistDTOList.forEach(metricHistDTO -> {
            if (metricHistDTO.getGreenLightEfficiency() != null && metricHistDTO.getGreenLightEfficiency() < 0.8) {
                metricHistDTO.setEmptyDischarges(1);
            }
        });

        List<SceneMetricsDetailVO> res = new ArrayList<>();
        if (CollectionUtil.isEmpty(metricHistDTOList)) {
            return res;
        }

        res = buildMetricsList(metricHistDTOList, minutes, metricCodes);
        return res;
    }

    @Override
    public SceneEvaluateMetricsTurnVO metricsTurn(MetricsTurnBO bo) throws ParseException {
        Integer status = bo.getStatus();
        String crossId = bo.getCrossId();
        Integer dir = bo.getDir();
        Date day = bo.getDay();
        String hourMinute = bo.getHourMinute();
        // 构造开始时间结束时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        String todayDateString = sdf.format(day); // Get today's date
        String startTimeString = todayDateString + " " + hourMinute + ":00";
        Date startTime = dateTimeFormatter.parse(startTimeString);

        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);
        cal.add(Calendar.MINUTE, 4);
        cal.add(Calendar.SECOND, 59);
        Date endTime = cal.getTime();
        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (endTime.getTime() / 1000);

        // 查询该时段内转向级别数据
        List<CrossTurnDataHistPO> crossTurnDataHistPOList =
                crossTurnDataHistMapper.selectByCrossIdAndDir(crossId, dir, endTimeStamp, startTimeStamp);
        Map<String, List<CrossTurnDataHistPO>> groupedByTurn = crossTurnDataHistPOList.stream()
                .collect(Collectors.groupingBy(CrossTurnDataHistPO::getTurnType));

        SceneEvaluateMetricsTurnVO res = new SceneEvaluateMetricsTurnVO();
        // 获取延误
        double v = crossTurnDataHistPOList.stream()
                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                .average()
                .orElse(0);
        Integer delayTime = (int)v;
        Integer level = calcCongestionLevel(delayTime);
        res.setLevel(level);

        List<SceneEvaluateMetricsTurnVO.TurnListElement> elementList = new ArrayList<>();
        for (Map.Entry<String, List<CrossTurnDataHistPO>> entry : groupedByTurn.entrySet()) {
            SceneEvaluateMetricsTurnVO.TurnListElement element = buildTurnListElement(entry, status);
            elementList.add(element);
        }
        res.setTurnList(elementList);
        return res;
    }

    private SceneEvaluateMetricsTurnVO.TurnListElement buildTurnListElement(
            Map.Entry<String, List<CrossTurnDataHistPO>> entry, Integer status) {
        SceneEvaluateMetricsTurnVO.TurnListElement element = new SceneEvaluateMetricsTurnVO.TurnListElement();

        String turn = entry.getKey();
        element.setTurn(turn);
        List<CrossTurnDataHistPO> poList = entry.getValue();
        // 获取流量
        OptionalDouble optionalAverageFlow = poList.stream()
                .filter(Objects::nonNull)
                .mapToInt(CrossTurnDataHistPO::getFlow)
                .average();
        int averageFlow = 0;
        if (optionalAverageFlow.isPresent()) {
            averageFlow = (int) Math.round(optionalAverageFlow.getAsDouble());
        }
        element.setFlow(averageFlow);
        // 动态指标
        element.setMetricsMap(buildTurnMetricMap(poList, status));

        return element;
    }

    private Map<String, Integer> buildTurnMetricMap(List<CrossTurnDataHistPO> poList, Integer status) {
        Map<String, Integer> res = new HashMap<>();

        if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            buildTurnCongestionRes(res, poList);
        } else if (Objects.equals(status, CrossStatusEnum.UNBALANCE.getCode())) {
            buildTurnUnbalanceRes(res, poList);
        } else if (Objects.equals(status, CrossStatusEnum.SPILLOVER.getCode())) {
            buildTurnSpilloverRes(res, poList);
        }

        return res;
    }

    private void buildTurnSpilloverRes(Map<String, Integer> res, List<CrossTurnDataHistPO> poList) {
        // 最大排队长度
        double maxQueueLength = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                .max()
                .orElse(0.0);
        int maxQueueLengthInt = (int) (Math.round(maxQueueLength));
        res.put(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode(), maxQueueLengthInt);
        // 平均延误
        double maxDelayTime = poList.stream()
                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                .average()
                .orElse(0.0);
        int maxDelayTimeInt = (int) (Math.round(maxDelayTime));
        res.put(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode(), maxDelayTimeInt);
        // 停车次数
        double maxStopTimes = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                .average()
                .orElse(0.0);
        int maxStopTimesInt = (int) (Math.round(maxStopTimes));
        res.put(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode(), maxStopTimesInt);
        // 溢流率
        double maxEffusionRate = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getEffusionRate)
                .average()
                .orElse(0.0);
        int maxEffusionRateInt = (int) (Math.round(maxEffusionRate * 100));
        res.put(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode(), maxEffusionRateInt);
    }

    private void buildTurnUnbalanceRes(Map<String, Integer> res, List<CrossTurnDataHistPO> poList) {
        // 不停车通过率
        double maxNoStopRate = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getNoStopRate)
                .average()
                .orElse(0.0);
        int maxNoStopRateInt = (int) (Math.round(maxNoStopRate * 100));
        res.put(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode(), maxNoStopRateInt);
        // N次停车通过率
        double maxOneStopRate = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                .average()
                .orElse(0.0);
        int maxOneStopRateInt = (int) (Math.round(maxOneStopRate * 100));
        res.put(StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode(), maxOneStopRateInt);
        // 空放次数-绿灯有效利用率小于80%
        long emptyDischarges = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                .filter(efficiency -> efficiency < 0.8)
                .count();
        res.put(StrategyAndMetricsEnum.Metrics.EMPTY_DISCHARGES.getCode(), (int)emptyDischarges);
        // 绿灯有效利用率
        double maxGreenLightEfficiency = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                .average()
                .orElse(0.0);
        int maxGreenLightEfficiencyInt = (int) (Math.round(maxGreenLightEfficiency * 100));
        res.put(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode(), maxGreenLightEfficiencyInt);
    }

    private void buildTurnCongestionRes(Map<String, Integer> res, List<CrossTurnDataHistPO> poList) {
        // N次停车通过率
        double maxOneStopRate = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                .average()
                .orElse(0.0);
        int maxOneStopRateInt = (int) (Math.round(maxOneStopRate * 100));
        res.put(StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode(), maxOneStopRateInt);
        // 平均速度
        double maxSpeed = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getSpeed)
                .average()
                .orElse(0.0);
        int maxSpeedInt = (int) (Math.round(maxSpeed));
        res.put(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode(), maxSpeedInt);
        // 平均延误
        double maxDelayTime = poList.stream()
                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                .average()
                .orElse(0.0);
        int maxDelayTimeInt = (int) (Math.round(maxDelayTime));
        res.put(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode(), maxDelayTimeInt);
        // 停车次数
        double maxStopTimes = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                .average()
                .orElse(0.0);
        int maxStopTimesInt = (int) (Math.round(maxStopTimes));
        res.put(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode(), maxStopTimesInt);
        // 最大排队长度
        double maxQueueLength = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                .max()
                .orElse(0.0);
        int maxQueueLengthInt = (int) (Math.round(maxQueueLength));
        res.put(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode(), maxQueueLengthInt);
        // 溢流率
        double maxEffusionRate = poList.stream()
                .mapToDouble(CrossTurnDataHistPO::getEffusionRate)
                .average()
                .orElse(0.0);
        int maxEffusionRateInt = (int) (Math.round(maxEffusionRate * 100));
        res.put(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode(), maxEffusionRateInt);
    }

    private List<SceneMetricsDetailVO> buildMetricsList(
            List<MetricHistDTO> metricHistDTOList, Integer minutes, List<String> metricCodes) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        List<SceneMetricsDetailVO> res = new ArrayList<>();
        // 按时间段分组
        Map<String, List<MetricHistDTO>> groupedByTime = metricHistDTOList.stream()
                .collect(Collectors.groupingBy(metricHistDTO -> {
                    long timestampInMillSeconds = metricHistDTO.getBatchTime() * 1000L;
                    Date date = new Date(timestampInMillSeconds);
                    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                    int minuteOfDay = date.getMinutes() + date.getHours() * 60;
                    int roundedMinute = (minuteOfDay / minutes) * minutes;
                    date.setHours(roundedMinute / 60);
                    date.setMinutes(roundedMinute % 60);
                    return sdf.format(date);
                }));
        // 获取时间段数组
        List<String> minuteSectionArray = TimeArrayUtil.getMinuteSectionArray(minutes);
        // 计算指标
        for (String section : minuteSectionArray) {
            List<MetricHistDTO> dtoList = groupedByTime.get(section);
            for (String metricCode : metricCodes) {
                SceneMetricsDetailVO crossMetrics = new SceneMetricsDetailVO();
                crossMetrics.setMetricTime(section);
                crossMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.getNameByCode(metricCode));
                if (CollectionUtil.isNotEmpty(dtoList)) {
                    int metricValue = 0;
                    for (MetricHistDTO metricHistDTO : dtoList) {
                        // 获取field名称
                        String fieldName = StrategyAndMetricsEnum.Metrics.getFieldByCode(metricCode);
                        // 首字母大写，以匹配JavaBean规范中的get方法名称
                        String capitalizedFieldName = fieldName.substring(0, 1).toUpperCase()
                                + fieldName.substring(1);
                        // 获取get方法
                        Method method = MetricHistDTO.class.getMethod("get" + capitalizedFieldName);
                        // 调用方法并获取返回值
                        Object value = method.invoke(metricHistDTO);
                        double valDouble = ((Number) value).doubleValue();;

                        if (fieldName.equals(StrategyAndMetricsEnum.Metrics.STOP_RATE.getField())
                                || fieldName.equals(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getField())
                                || fieldName.equals(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getField())
                                || fieldName.equals(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getField())) {
                            valDouble *= 100;
                        }
                        int valInt = (int)valDouble;
                        metricValue += valInt;
                    }
                    int size = dtoList.size();
                    if (!Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.EMPTY_DISCHARGES.getCode())) {
                        crossMetrics.setMetricValue(metricValue / size);
                    } else {
                        crossMetrics.setMetricValue(metricValue);
                    }
                }
                res.add(crossMetrics);
            }
        }
        return res;
    }

    private List<String> getTimeline(Date startTime, Date newEndTime) {
        List<String> res = new ArrayList<>();

        Calendar startCal = Calendar.getInstance();
        startCal.setTime(startTime);
        int minute = startCal.get(Calendar.MINUTE);
        int remainder = minute % 5;
        startCal.add(Calendar.MINUTE, -remainder);

        Calendar endCal = Calendar.getInstance();
        endCal.setTime(newEndTime);
        minute = endCal.get(Calendar.MINUTE);
        remainder = (5 - (minute % 5)) % 5;
        endCal.add(Calendar.MINUTE, remainder);

        for (Calendar cal = (Calendar) startCal.clone();
             cal.before(endCal) || cal.equals(endCal);
             cal.add(Calendar.MINUTE, 5)) {
            res.add(HOUR_SDF.format(cal.getTime()));
        }

        return res;
    }

    private List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> buildCrossSpilloverMetrics(
            List<CrossDataHistPO> sceneCrossDataHistPOList) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> res = new ArrayList<>();
        // 最大排队长度
        double maxQueueLength = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getQueueLength)
                .max()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH, maxQueueLength));
        // 平均延误
        double avgDelayTime = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getDelayTime)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY, avgDelayTime));
        // 平均停车次数
        double avgStopTimes = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getStopTimes)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.STOP_TIMES, avgStopTimes));
        // 溢流率
        double avgEffusionRate = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getEffusionRate)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE, avgEffusionRate * 100));

        return res;
    }

    private List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> buildCrossUnbalanceMetrics(
            String crossId, Date startTime, Date newEndTime, List<CrossDataHistPO> sceneCrossDataHistPOList) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> res = new ArrayList<>();
        List<CrossDirDataHistPO> crossDirDataHistPOList = getCrossDirDataHistPOList(startTime, newEndTime, crossId);

        // 用方向数据计算不停车通过率
        double avgNoStopRate = crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE, avgNoStopRate * 100));
        // 用方向数据计算N次停车通过率
        double maxOneStopRate = crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.STOP_RATE, maxOneStopRate * 100));
        // 空放次数-绿灯有效利用率小于80%
        long emptyDischarges = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getGreenLightEfficiency)
                .filter(efficiency -> efficiency < 0.8)
                .count();
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.EMPTY_DISCHARGES, emptyDischarges));
        // 绿灯有效利用率
        double avgGreenLightEfficiency = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getGreenLightEfficiency)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY,
                avgGreenLightEfficiency * 100));
        // 负载均衡度
        double avgLoadBalance = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getLoadBalance)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.LOAD_BALANCE, avgLoadBalance));

        return res;
    }

    private List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> buildCrossCongestionMetrics(
            String crossId, Date startTime, Date newEndTime, List<CrossDataHistPO> sceneCrossDataHistPOList) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> res = new ArrayList<>();

        // 从方向数据计算N次停车通过率
        List<CrossDirDataHistPO> crossDirDataHistPOList = getCrossDirDataHistPOList(startTime, newEndTime, crossId);
        double maxOneStopRate = crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.STOP_RATE, maxOneStopRate * 100));

        double avgSpeed = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getSpeed)
                .average()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED, avgSpeed));

        int maxDelayTime = sceneCrossDataHistPOList.stream()
                .mapToInt(CrossDataHistPO::getDelayTime)
                .max()
                .orElse(0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY, maxDelayTime));

        double maxStopTimes = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getStopTimes)
                .max()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.STOP_TIMES, maxStopTimes));

        double maxQueueLength = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getQueueLength)
                .max()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH, maxQueueLength));

        double maxEffusionRate = sceneCrossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getEffusionRate)
                .max()
                .orElse(0.0);
        res.add(buildMetrics(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE, maxEffusionRate * 100));

        return res;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildMetrics(
            StrategyAndMetricsEnum.Metrics metricEnum, double value) {
        int roundedValue = (int) Math.round(value);
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics metric = new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        metric.setMetricCode(metricEnum.getCode());
        metric.setMetricUnit(metricEnum.getUnit());
        metric.setMetricName(metricEnum.getDescription());
        metric.setMetricValue(roundedValue);
        return metric;
    }


    private List<CrossDirDataHistPO> getCrossDirDataHistPOList(Date startTime, Date newEndTime, String crossId) {
        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (newEndTime.getTime() / 1000);
        List<CrossDirDataHistPO> res = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        return res;
    }

    private Integer calcCrossFlow(List<CrossDataHistPO> crossDataHistPOList) {
        int max = crossDataHistPOList.stream()
                .mapToInt(CrossDataHistPO::getFlow)
                .max()
                .orElse(0);
        return max;
    }

    private Double calcCrossSaturation(List<CrossDataHistPO> crossDataHistPOList) {
        double max = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getSturation)
                .max()
                .orElse(0.0);
        return max;
    }

    private List<CrossDataHistPO> getCrossDataHistPOList(Date startTime, Date newEndTime, String crossId) {
        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (newEndTime.getTime() / 1000);
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        return crossDataHistPOList;
    }

    private Integer calcCrossDelayTime(List<CrossDataHistPO> crossDataHistPOList) {
        int max = crossDataHistPOList.stream()
                .mapToInt(CrossDataHistPO::getDelayTime)
                .max()
                .orElse(0);
        return max;
    }



    private List<String> buildPatternNames(List<BaseCrossSchemePO> matchedSchemes) {
        List<String> res = new ArrayList<>();
        for (BaseCrossSchemePO matchedScheme : matchedSchemes) {
            res.add(matchedScheme.getName());
        }
        return res;
    }

    private List<Integer> buildDirs(String congestionDirs) {
        List<Integer> res = new ArrayList<>();
        String[] split = congestionDirs.split(",");
        for (String s : split) {
            res.add(Integer.parseInt(s));
        }
        return res;
    }

    private Integer calcCongestionLevel(Integer delayTime) {
        Integer res = null;
        if (delayTime >= 145) {
            res = 3;
        } else if (delayTime >= 100) {
            res = 2;
        } else if (delayTime >= 55) {
            res = 1;
        } else {
            res = 0;
        }
        return res;
    }

    public List<SceneEvaluateAbnormalDistributeVO.TimeDateDistribution> buildFinalTimeDistributionList(
            List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList) {
        List<SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution> finalTimeDistributionList = new ArrayList<>();

        for (SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution : timeDistributionList) {
            Date problemDate = timeDistribution.getProblemDate();

            for (RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus : timeDistribution.getProblemStatusList()) {
                Integer status = problemStatus.getStatus();
                Date startTime = problemStatus.getStartTime();
                Date endTime = problemStatus.getEndTime();
                // 生成 FinalTimeDistribution 对象
                for (Date time = getStartTimeOfDay(); time.before(getEndTimeOfDay()); time = addFiveMinutes(time)) {
                    if (isTimeIntersecting(time, addFiveMinutes(time), startTime, endTime)) {
                        SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution finalTimeDistribution =
                                new SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution();
                        finalTimeDistribution.setProblemDate(problemDate);
                        finalTimeDistribution.setStatus(status);
                        // 将 time 中的年、月、日设置为 problemDate 中的年、月、日
                        Calendar calTime = Calendar.getInstance();
                        Calendar calProblemDate = Calendar.getInstance();
                        calTime.setTime(time);
                        calProblemDate.setTime(problemDate);

                        calTime.set(Calendar.YEAR, calProblemDate.get(Calendar.YEAR));
                        calTime.set(Calendar.MONTH, calProblemDate.get(Calendar.MONTH));
                        calTime.set(Calendar.DAY_OF_MONTH, calProblemDate.get(Calendar.DAY_OF_MONTH));

                        Date newTime = calTime.getTime();
                        finalTimeDistribution.setDistributionTime(newTime);
                        finalTimeDistributionList.add(finalTimeDistribution);
                    }
                }
            }
            // 填满 finalTimeDistributionList
            // 生成时间列表
            List<Date> timeList = new ArrayList<>();
            Calendar cal = Calendar.getInstance();
            cal.setTime(problemDate);
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MILLISECOND, 0);

            for (int i = 0; i < 24 * 60; i += 5) {
                timeList.add(cal.getTime());
                cal.add(Calendar.MINUTE, 5);
            }

            for (Date time : timeList) {
                boolean exists = finalTimeDistributionList.stream()
                        .anyMatch(item -> item.getDistributionTime().equals(time)
                                && isSameDay(item.getProblemDate(), problemDate));
                if (!exists) {
                    SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution newItem =
                            new SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution();
                    newItem.setProblemDate(problemDate);
                    newItem.setDistributionTime(time);
                    newItem.setStatus(null);
                    finalTimeDistributionList.add(newItem);
                }
            }
        }
        // 重新排序
        finalTimeDistributionList
                .sort(Comparator.comparing(SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution::getProblemDate)
                .thenComparing(SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution::getDistributionTime));
        // 生成时间日期分布
        List<SceneEvaluateAbnormalDistributeVO.TimeDateDistribution> timeDateDistributionList = new ArrayList<>();
        List<String> minuteSectionArray = TimeArrayUtil.getMinuteSectionArray(5);
        for (String timeStr : minuteSectionArray) {
            SceneEvaluateAbnormalDistributeVO.TimeDateDistribution timeDateDistribution =
                    new SceneEvaluateAbnormalDistributeVO.TimeDateDistribution();
            timeDateDistribution.setHourMinute(timeStr);
            timeDateDistribution.setStatusByDateList(buildStatusByDateList(timeStr, finalTimeDistributionList));
            timeDateDistributionList.add(timeDateDistribution);
        }
        return timeDateDistributionList;
    }

    private List<SceneEvaluateAbnormalDistributeVO.StatusByDate> buildStatusByDateList(
            String timeStr, List<SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution> finalTimeDistributionList) {
        List<SceneEvaluateAbnormalDistributeVO.StatusByDate> res = new ArrayList<>();

        for (SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd : finalTimeDistributionList) {
            Date problemDate = ftd.getProblemDate();
            Date distributionTime = ftd.getDistributionTime();
            String format = HOUR_SDF.format(distributionTime);
            if (Objects.equals(timeStr, format)) {
                SceneEvaluateAbnormalDistributeVO.StatusByDate inRes = getRecordInRes(res, problemDate);
                // 如果 res 中没有记录
                if (ObjectUtil.isEmpty(inRes)) {
                    SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate = buildStatusByDate(ftd, problemDate);
                    res.add(statusByDate);
                } else {
                    modifyRecordInRes(ftd, inRes);
                }
            }
        }
        return res;
    }

    private void modifyRecordInRes(
            SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd,
            SceneEvaluateAbnormalDistributeVO.StatusByDate inRes) {
        Integer status = ftd.getStatus();
        if (Objects.equals(status, CrossStatusEnum.UNBALANCE.getCode())) {
            inRes.setIsUnbalance(1);
        } else if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            inRes.setIsCongestion(1);
        } else if (Objects.equals(status, CrossStatusEnum.SPILLOVER.getCode())) {
            inRes.setIsSpillover(1);
        }
    }

    @NotNull
    private SceneEvaluateAbnormalDistributeVO.StatusByDate buildStatusByDate(
            SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd, Date problemDate) {
        SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate =
                new SceneEvaluateAbnormalDistributeVO.StatusByDate();
        statusByDate.setProblemDate(problemDate);
        modifyRecordInRes(ftd, statusByDate);
        return statusByDate;
    }

    private SceneEvaluateAbnormalDistributeVO.StatusByDate getRecordInRes(
            List<SceneEvaluateAbnormalDistributeVO.StatusByDate> res, Date problemDate) {
        for (SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate : res) {
            Date problemDateInRes = statusByDate.getProblemDate();
            // 创建Calendar实例
            Calendar cal1 = Calendar.getInstance();
            Calendar cal2 = Calendar.getInstance();
            // 设置Calendar实例的时间
            cal1.setTime(problemDate);
            cal2.setTime(problemDateInRes);
            // 比较两个日期是否相等（忽略时间部分）
            boolean isSameDay = cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                    cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR);
            if (isSameDay) {
                return statusByDate;
            }
        }
        return null;
    }

    private boolean isSameDay(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        Calendar cal2 = Calendar.getInstance();
        cal1.setTime(date1);
        cal2.setTime(date2);
        return cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR);
    }

    // 辅助方法：判断两个时间段是否相交
    public boolean isTimeIntersecting(Date eventStartTime, Date eventEndTime,
                                             Date sectionStartTime, Date sectionEndTime) {
        Calendar cal = java.util.Calendar.getInstance();

        cal.setTime(eventStartTime);
        int eventStartHour = cal.get(java.util.Calendar.HOUR_OF_DAY);
        int eventStartMinute = cal.get(java.util.Calendar.MINUTE);

        cal.setTime(eventEndTime);
        int eventEndHour = cal.get(java.util.Calendar.HOUR_OF_DAY);
        int eventEndMinute = cal.get(java.util.Calendar.MINUTE);

        cal.setTime(sectionStartTime);
        int sectionStartHour = cal.get(Calendar.HOUR_OF_DAY);
        int sectionStartMinute = cal.get(Calendar.MINUTE);

        cal.setTime(sectionEndTime);
        int sectionEndHour = cal.get(Calendar.HOUR_OF_DAY);
        int sectionEndMinute = cal.get(Calendar.MINUTE);

        // 将小时和分钟转换为分钟数以便比较
        int eventStartInMinutes = eventStartHour * 60 + eventStartMinute;
        int eventEndInMinutes = eventEndHour * 60 + eventEndMinute;
        int sectionStartInMinutes = sectionStartHour * 60 + sectionStartMinute;
        int sectionEndInMinutes = sectionEndHour * 60 + sectionEndMinute;

        // 判断两个时间段是否相交
        return (eventStartInMinutes < sectionEndInMinutes && eventEndInMinutes > sectionStartInMinutes) ||
                (sectionStartInMinutes < eventEndInMinutes && sectionEndInMinutes > eventStartInMinutes);
    }

    // 辅助方法：获取当天的开始时间（00:00）
    private static Date getStartTimeOfDay() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    // 辅助方法：获取当天的结束时间（23:59）
    private static Date getEndTimeOfDay() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar.getTime();
    }

    // 辅助方法：给时间加上5分钟
    private static Date addFiveMinutes(Date time) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time);
        calendar.add(Calendar.MINUTE, 5);
        return calendar.getTime();
    }

    private static void setCounts(List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList,
                                  SceneEvaluateAbnormalDistributeVO vo) {
        int congestionDuration = 0;
        int congestionTimes = 0;
        int unbalanceDuration = 0;
        int unbalanceTimes = 0;
        int spilloverDuration = 0;
        int spilloverTimes = 0;
        for (SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution : timeDistributionList) {
            List<RunningEvaluateMetricsDetailVO.ProblemStatus> problemStatusList =
                    timeDistribution.getProblemStatusList();
            for (RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus : problemStatusList) {
                Integer status = problemStatus.getStatus();
                Integer durationMinutes = problemStatus.getDurationMinutes();
                if (status != null && status.equals(CrossStatusEnum.CONGESTION.getCode())) {
                    congestionTimes++;
                    congestionDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.UNBALANCE.getCode())) {
                    unbalanceTimes++;
                    unbalanceDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.SPILLOVER.getCode())) {
                    spilloverTimes++;
                    spilloverDuration += durationMinutes;
                }
            }
        }
        vo.setCongestionTimes(congestionTimes);
        vo.setUnbalanceTimes(unbalanceTimes);
        vo.setSpilloverTimes(spilloverTimes);
        vo.setCongestionDuration(congestionDuration);
        vo.setUnbalanceDuration(unbalanceDuration);
        vo.setSpilloverDuration(spilloverDuration);
    }

    private List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> buildTimeDistributionList(
            String crossId, Date startDate, Date endDate) {
        List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> res = new ArrayList<>();
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(startDate);
        end.setTime(endDate);
        // 使 endDate 包含在内
        end.add(Calendar.DATE, 1);

        for (Date date = start.getTime(); start.before(end); start.add(Calendar.DATE, 1), date = start.getTime()) {
            SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution =
                    new SceneEvaluateAbnormalDistributeVO.TimeDistribution();
            timeDistribution.setProblemDate(date);
            // 获取问题列表
            // 路口级别全量数据
            int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
            int endStamp = (int) (endDate.getTime() / 1000);
            List<MetricHistDTO> crossDTOList = crossDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
            // 过滤有问题的记录
            List<MetricHistDTO> filteredList = crossDTOList.stream()
                    .filter(metricHistDTO -> metricHistDTO.getStatus() != null && !metricHistDTO.getStatus().equals(0))
                    .collect(Collectors.toList());
            timeDistribution.setProblemStatusList(runningEvaluateService.buildProblemStatusList(filteredList));
            res.add(timeDistribution);
        }
        return res;
    }
}
